import {
  Body,
  Controller,
  Delete,
  Get,
  Headers,
  InternalServerErrorException,
  NotFoundException,
  Param,
  Post,
  Put,
  Query,
  UnauthorizedException,
  UseGuards,
} from '@nestjs/common';
import { BooksService } from './books.service';
import { CurrentUser } from 'src/core/users/decorators/current-user.decorator';
import { User } from 'src/core/users/user.entity';
import { StripeService } from 'src/core/stripe/stripe.service';
import { config } from 'dotenv';
import { JwtGuard } from 'src/core/auth/guards/jwt-auth.guard';
import { JwtService } from '@nestjs/jwt';
import { MyBooksService } from '../my-books/books.service';

config();

@Controller('books')
export class BooksController {
  adminEmail = `${process.env.ADMIN_EMAIL}`;
  constructor(
    private booksService: BooksService,
    private stripeService: StripeService,
    private myBooksService: MyBooksService,
    private jwtService: JwtService,
  ) {}

  @Post()
  @UseGuards(JwtGuard)
  async addBook(
    @Body()
    body: any,
    @Headers() headers,
  ) {
    const book = await this.booksService.addBook(body);
    if (book) return book;
    return new InternalServerErrorException('Error creating book!');
  }

  @Put('publish/:id')
  @UseGuards(JwtGuard)
  async publishBook(
    @CurrentUser() user: User,
    @Body() body,
    @Param('id') bookId,
  ) {
    return this.booksService.publishBook(bookId, body.published);
  }

  @Get('checkout')
  @UseGuards(JwtGuard)
  async Checkout(
    @CurrentUser() user: User,
    @Query('ids') ids,
    @Headers() headers,
  ) {
    const idsAr = this.stripeService.getIdsAsArray(ids);
    const results = await this.booksService.getBooks(idsAr);
    if (results && results[0].length > 0) {
      const items = results[0].map((t) => ({
        id: t.id,
        name: t.title,
        price: t.price,
        imageUrl: t.frontImageUrl,
        quantity: 1,
      }));
      return this.stripeService.checkout(user.id, items, headers);
    }
    throw new NotFoundException('Books not found!');
  }

  @Post('/filter')
  getFiltered(
    @Query('take') take: number,
    @Query('skip') skip: number,
    @Body() body,
  ) {
    return this.booksService.getFiltered(body.filter, take, skip);
  }

  @Get('/all')
  async getAll(@Query('take') take: number, @Query('skip') skip: number) {
    const r = await this.booksService.getAll(take, skip);
    return r;
  }

  @Get('/all-series')
  async getAllSeries(
    @Query('series') series: string,
    @Query('take') take: number,
    @Query('skip') skip: number,
  ) {
    const r = await this.booksService.getAllSeries(series, take, skip);
    return r;
  }

  @Get('/all-tags')
  async getAllTag(
    @Query('tags') tags: string,
    @Query('take') take: number,
    @Query('skip') skip: number,
  ) {
    const r = await this.booksService.getAllTag(tags, take, skip);
    return r;
  }

  @Get('/admin-published-books')
  async getAdminPublishedBooks(
    @Query('take') take: number,
    @Query('skip') skip: number,
  ) {
    const r = await this.booksService.getAllPublished(take, skip);
    return r;
  }

  @Get('/admin-not-published-books')
  async getAdminNotPublishedBooks(
    @Query('take') take: number,
    @Query('skip') skip: number,
  ) {
    const r = await this.booksService.getAllNotPublished(take, skip);
    return r;
  }

  @Get(':id')
  async getBook(@Headers() headers, @Param('id') id: number) {
    const book = await this.booksService.getBook(id);
    if (book) {
      const auth = headers?.authorization;
      if (auth) {
        const jwt = auth.replace('Bearer ', '');
        const obj: any = this.jwtService.decode(jwt, { json: true }) as {
          uuid: string;
        };
        if (obj && obj.sub) {
          if (obj.sub.email === this.adminEmail) {
            return book;
          }
          const exists = await this.myBooksService.isOneOfMyBooks(
            obj.sub.id,
            id,
          );
          if (exists) {
            return book;
          } else {
            const { pdfUrl, epubUrl, ...newBook } = book;
            return newBook;
          }
        }
      }
      const { pdfUrl, epubUrl, ...newBook } = book;
      return newBook;
    }
  }

  @Put(':id')
  @UseGuards(JwtGuard)
  updateBook(@CurrentUser() user: User, @Param('id') id: number, @Body() book) {
    if (user && user.email === this.adminEmail)
      return this.booksService.updateBook(id, book);
    throw new UnauthorizedException(
      'You are not authorized to update the book',
    );
  }

  @Delete('/:id')
  @UseGuards(JwtGuard)
  deleteBook(@CurrentUser() user: User, @Param('id') bookId: number) {
    if (user && user.email === this.adminEmail) {
      return this.booksService.removeBook(bookId);
    }
    throw new UnauthorizedException(
      'You do not have a permission to delete the book',
    );
  }
}
