import {
  Injectable,
  InternalServerErrorException,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { DeepPartial, ILike, In, Repository } from 'typeorm';
import { Book } from './book.entity';

@Injectable()
export class BooksService {
  constructor(@InjectRepository(Book) private repo: Repository<Book>) {}
  async addBook(dto: DeepPartial<Book>) {
    const book = this.repo.create(dto);
    const saved = await this.repo.save(book);
    if (saved) return saved;
    else throw new InternalServerErrorException('Error Saving Book');
  }

  async getBookFile(bookId: number, type: string) {
    const book = await this.getBook(bookId);
    if (book) {
      let bookObj: any = {
        filename: book.title,
      };
      switch (type) {
        case 'pdf':
          bookObj.filePath = book.pdfUrl;
          bookObj.fileType = 'pdf';
          break;
        case 'epub':
          bookObj.filePath = book.epubUrl;
          bookObj.fileType = 'epub';
          break;
        case 'zip':
          bookObj.filePath = book.zipUrl;
          bookObj.fileType = 'zip';
          break;
      }
      return bookObj;
    }
    throw new NotFoundException('Book not found!');
  }

  async publishBook(bookId: number, published: boolean) {
    const book = await this.repo.findOne({
      where: {
        id: bookId,
      },
    });
    if (book) {
      book.published = published;
      return this.repo.save(book);
    }
    throw new NotFoundException('Book not found!');
  }

  async updateBook(bookId: number, body: DeepPartial<Book>) {
    const obj = await this.repo.findOne({
      where: {
        id: bookId,
      },
    });
    if (obj) {
      return this.repo.update(
        {
          id: bookId,
        },
        body,
      );
    }
    throw new NotFoundException('Book not found');
  }

  getFiltered(filter: any, take: number, skip: number) {
    return this.repo.findAndCount({
      where: {
        title: filter.title ? ILike(`%${filter.title}%`) : undefined,
        subtitle: filter.subtitle ? ILike(`%${filter.subtitle}%`) : undefined,
        tags: filter.tags ? ILike(`%${filter.tags}%`) : undefined,
        series: filter.series ? ILike(`%${filter.series}%`) : undefined,
        published: true,
      },
      order: {
        publishDate: {
          direction: 'DESC',
        },
      },
      take,
      skip,
    });
  }

  findOne(id: number) {
    if (!id) {
      return null;
    }
    return this.repo.findOneBy({ id });
  }

  async removeBook(id: number) {
    const book = await this.repo.findOne({
      where: {
        id,
      },
    });
    if (book) return this.repo.remove(book);
    throw new NotFoundException('Book not found!');
  }

  getAllForIds(ids: number[], take: number, skip: number) {
    return this.repo.findAndCount({
      where: {
        id: In(ids),
      },
      order: {
        publishDate: {
          direction: 'DESC',
        },
      },
      take,
      skip,
    });
  }

  getAll(take: number, skip: number) {
    return this.repo.findAndCount({
      where: {
        published: true,
      },
      order: {
        publishDate: {
          direction: 'DESC',
        },
      },
      take,
      skip,
    });
  }

  getAllSeries(series: string, take: number, skip: number) {
    return this.repo.findAndCount({
      where: {
        published: true,
        series,
      },
      order: {
        publishDate: {
          direction: 'DESC',
        },
      },
      take,
      skip,
    });
  }

  getTagsAsStringArray(tags: string) {
    let tgsAr = [];
    if (tags) {
      if (tags.includes(',')) {
        tgsAr = tags.split(',').map((t) => t.trim());
      } else {
        tgsAr = [tags.trim()];
      }
    }
    return tgsAr;
  }

  getAllTag(tags: string, take: number, skip: number) {
    let tgsAr = this.getTagsAsStringArray(tags);
    return this.repo.findAndCount({
      where: {
        published: true,
        tags: In(tgsAr),
      },
      order: {
        publishDate: {
          direction: 'DESC',
        },
      },
      take,
      skip,
    });
  }

  getAllTopic(topic: number, take: number, skip: number) {
    return this.repo.findAndCount({
      where: {
        published: true,
        topic,
      },
      order: {
        publishDate: {
          direction: 'DESC',
        },
      },
      take,
      skip,
    });
  }

  getAllTopics(topics: string, take: number, skip: number) {
    let topicsArr = topics ? topics?.split(',').map((t) => parseInt(t)) : [];
    return this.repo.findAndCount({
      where: {
        published: true,
        topic: In(topicsArr),
      },
      order: {
        publishDate: {
          direction: 'DESC',
        },
      },
      take,
      skip,
    });
  }

  getAllPublished(take: number, skip: number) {
    return this.repo.findAndCount({
      where: {
        published: true,
      },
      order: {
        publishDate: {
          direction: 'DESC',
        },
      },
      take,
      skip,
    });
  }

  getAllNotPublished(take: number, skip: number) {
    return this.repo.findAndCount({
      where: {
        published: false,
      },
      order: {
        publishDate: {
          direction: 'DESC',
        },
      },
      take,
      skip,
    });
  }

  getBook(id: number) {
    return this.findOne(id);
  }

  getBooks(ids: number[]) {
    return this.repo.findAndCount({
      where: {
        id: In(ids),
      },
    });
  }
}
