import { Component, OnInit } from '@angular/core';
import {
  FormBuilder,
  FormsModule,
  ReactiveFormsModule,
  Validators,
} from '@angular/forms';
import { InputTextModule } from 'primeng/inputtext';
import { DropdownModule } from 'primeng/dropdown';
import { ButtonModule } from 'primeng/button';
import {
  ConfirmationService,
  MessageService,
  SelectItemGroup,
} from 'primeng/api';
import { PanelModule } from 'primeng/panel';
import { DecimalPipe } from '@angular/common';
import { OverlayPanelModule } from 'primeng/overlaypanel';
import { StarComponent } from '../../components/star/star.component';
import { ToastModule } from 'primeng/toast';
import { loadStripe } from '@stripe/stripe-js/pure';
import { environment } from '../../../environments/environment';
import { DividerModule } from 'primeng/divider';
import { ImageUrlComponent } from '../../components/image-url/image-url.component';
import { InputTextareaModule } from 'primeng/inputtextarea';
import { AuthService } from '../../services/auth.service';
import { CharacterCountComponent } from '../../components/character-count/character-count.component';
import { first } from 'rxjs';
import { QuillModule } from 'ngx-quill';
import { ActivatedRoute, Router, RouterLink } from '@angular/router';
import { ConfirmDialogModule } from 'primeng/confirmdialog';
import { BreadcrumbComponent } from '../../components/breadcrumb/breadcrumb.component';
import { Book } from '../../data/book';
import { ToggleButtonModule } from 'primeng/togglebutton';
import { BooksService } from '../../services/books.service';
import { FeaturedBookService } from '../../services/featured-book.service';
import { CheckboxModule } from 'primeng/checkbox';
import { MAX_KEYPOINT_INPUT_TEXT_LENGTH } from '../../data/constants';
import { TOPICS } from '../../data/topics';

@Component({
  selector: 'app-post',
  standalone: true,
  imports: [
    ReactiveFormsModule,
    InputTextModule,
    DropdownModule,
    ButtonModule,
    PanelModule,
    FormsModule,
    DecimalPipe,
    OverlayPanelModule,
    StarComponent,
    ToastModule,
    DividerModule,
    ImageUrlComponent,
    InputTextareaModule,
    CharacterCountComponent,
    QuillModule,
    ConfirmDialogModule,
    BreadcrumbComponent,
    ToggleButtonModule,
    CheckboxModule,
  ],
  templateUrl: './add-book.component.html',
  styleUrl: './add-book.component.css',
  providers: [ConfirmationService, MessageService],
})
export class AddBookComponent implements OnInit {
  topicsList: SelectItemGroup[] = TOPICS;
  selectedTopic = {
    label: this.topicsList[0].items[0].label,
    value: this.topicsList[0].items[0].value,
  };
  items: any[] = [];
  max_keypoint_input_length = MAX_KEYPOINT_INPUT_TEXT_LENGTH;
  isAdmin = false;
  book!: Book;
  bookId!: number;
  form = this.fb.group({
    title: this.fb.control('', [Validators.required]),
    language: this.fb.control(''),
    publisher: this.fb.control(''),
    isbn: this.fb.control(''),
    subtitle: this.fb.control(''),
    author: this.fb.control('', [Validators.required]),
    keyPoint1: this.fb.control(''),
    keyPoint2: this.fb.control(''),
    keyPoint3: this.fb.control(''),
    publishDate: this.fb.control(new Date()),
    description: this.fb.control(''),
    folderPath: this.fb.control(''),
    darkCover: this.fb.control(false),
    frontImageUrl: this.fb.control(''),
    backImageUrl: this.fb.control('', []),
    sideImageUrl: this.fb.control('', []),
    printUrl: this.fb.control(''),
    pdfUrl: this.fb.control(''),
    epubUrl: this.fb.control(''),
    zipUrl: this.fb.control(''),
    price: this.fb.control(9.99),
    series: this.fb.control(''),
    published: this.fb.control(true),
    tags: this.fb.control(''),
    topic: this.fb.control(this.selectedTopic.value),
    order: this.fb.control(1),
    numberOfPages: this.fb.control(1),
  });

  constructor(
    private route: ActivatedRoute,
    private booksService: BooksService,
    private featuredBookService: FeaturedBookService,
    private fb: FormBuilder,
    private messagesService: MessageService,
    private authService: AuthService,
    private confirmationService: ConfirmationService,
    private router: Router
  ) {
    this.bookId = this.route.snapshot.params['id'];
  }

  ngOnInit(): void {
    this.items.push({
      label: 'Manage Books',
      link: '/manage-books',
    });
    if (this.book && this.bookId) {
      this.items.push({
        label: 'Edit',
        link: '',
      });
    } else {
      this.items.push({
        label: 'Add New',
        link: '',
      });
    }
    this.loadBook();
    this.authService.isAdmin.subscribe((r) => (this.isAdmin = r));
  }

  onFeature() {
    this.featuredBookService.setFeaturedBook(this.bookId).subscribe({
      next: async (r) => {
        this.messagesService.add({
          severity: 'info',
          summary: 'Featured',
          detail: 'Book featured successfully',
        });
      },
      error: (e) => {
        console.log(e);
        this.messagesService.add({
          severity: 'error',
          summary: 'Error',
          detail: e.error.message,
        });
      },
    });
  }

  loadBook() {
    if (this.bookId) {
      this.booksService
        .getBook(this.bookId)
        .pipe(first())
        .subscribe((book: any) => {
          this.book = book;
          const topicObj = this.topicsList
            .flatMap((t) => t.items)
            .find((t) => t.value === this.book.topic);
          this.selectedTopic = {
            label: topicObj?.label!,
            value: topicObj?.value,
          };
          const keyPoints = this.book.keyPoints?.split('|');
          const keyPoint1 = keyPoints[0] ?? '';
          const keyPoint2 = keyPoints[1] ?? '';
          const keyPoint3 = keyPoints[2] ?? '';
          const images = this.booksService.getImages(book.folderPath);
          const files = this.booksService.getFiles(book.folderPath);
          this.form = this.fb.group({
            title: this.fb.control(this.book.title, [Validators.required]),
            language: this.fb.control(this.book.language),
            publisher: this.fb.control(this.book.publisher),
            isbn: this.fb.control(this.book.isbn),
            subtitle: this.fb.control(this.book.subtitle),
            author: this.fb.control(this.book.author, [Validators.required]),
            keyPoint1: this.fb.control(keyPoint1),
            keyPoint2: this.fb.control(keyPoint2),
            keyPoint3: this.fb.control(keyPoint3),
            publishDate: this.fb.control(this.book.publishDate),
            description: this.fb.control(this.book.description),
            folderPath: this.fb.control(this.book.folderPath),
            darkCover: this.fb.control(this.book.darkCover),
            frontImageUrl: this.fb.control(images.front),
            backImageUrl: this.fb.control(images.back),
            sideImageUrl: this.fb.control(images.side),
            printUrl: this.fb.control(this.book.printUrl),
            pdfUrl: this.fb.control(this.book.pdfUrl),
            epubUrl: this.fb.control(this.book.epubUrl),
            zipUrl: this.fb.control(this.book.zipUrl),
            price: this.fb.control(this.book.price, [
              Validators.required,
              Validators.min(1),
            ]),
            series: this.fb.control(this.book.series),
            published: this.fb.control(this.book.published),
            tags: this.fb.control(this.book.tags),
            topic: this.fb.control(this.book.topic),
            order: this.fb.control(this.book.order),
            numberOfPages: this.fb.control(this.book.numberOfPages),
          });
        });
    }
  }

  onPublish() {
    if (this.bookId && this.book) {
      this.booksService
        .publishBook(this.bookId, this.form.value.published!)
        .subscribe();
    }
  }

  onSubmit() {
    const keyPoints =
      this.form.value.keyPoint1 +
      '|' +
      this.form.value.keyPoint2 +
      '|' +
      this.form.value.keyPoint3;
    const { keyPoint1, keyPoint2, keyPoint3, ...obj } = this.form.value;
    if (this.bookId && this.book) {
      const data = {
        id: this.bookId,
        ...obj,
        keyPoints,
      };
      this.booksService.updateBook(data).subscribe({
        next: async (r) => {
          this.messagesService.add({
            severity: 'info',
            summary: 'Update',
            detail: 'Book updated successfully',
          });
        },
        error: (e) => {
          console.log(e);
          this.messagesService.add({
            severity: 'error',
            summary: 'Error',
            detail: e.error.message,
          });
        },
      });
    } else {
      const data = {
        ...obj,
        keyPoints,
      };
      this.booksService
        .addBook(data)
        .pipe(first())
        .subscribe({
          next: async (r: any) => {
            this.bookId = r.id;
            this.book = r;
            this.messagesService.add({
              severity: 'info',
              summary: 'Successful',
              detail: 'Book added successfully.',
            });
            // if (this.isAdmin) {
            //   this.messagesService.add({
            //     severity: 'info',
            //     summary: 'Successful',
            //     detail: 'Ad created successfully.',
            //   });
            //   this.form.patchValue({
            //     title: '',
            //     subtitle: '',
            //     author: '',
            //     description: '',
            //     imageUrl: '',
            //     mainUrl: '',
            //     discountUrl: '',
            //     freeUrl: '',
            //     numberOfDays: 365,
            //     topic: 1001,
            //     type: 1,
            //   });
            // } else {
            // const session = r as any;
            // const stripe = await loadStripe(environment.STRIPE_PK);
            // const result = await stripe?.redirectToCheckout({
            //   sessionId: session.id,
            // });
            // if (result?.error) {
            //   console.log(result?.error);
            // }
            // }
          },
          error: (e) => {
            console.log(e);
            this.messagesService.add({
              severity: 'error',
              summary: 'Error',
              detail: e.error.message,
            });
          },
        });
    }
  }

  onChangeTopic() {
    const obj = this.topicsList
      .flatMap((t) => t.items)
      .find((t) => t.value === this.form.value.topic);
    this.selectedTopic = {
      label: obj?.label,
      value: obj?.value,
    };
  }

  onFolderPath() {
    const front =
      environment.DOWNLOAD_BASE_URL +
      '/' +
      this.form.value.folderPath +
      '/cover/' +
      'front-313.jpg';
    const back =
      environment.DOWNLOAD_BASE_URL +
      '/' +
      this.form.value.folderPath +
      '/cover/' +
      'back-313.jpg';

    const side =
      environment.DOWNLOAD_BASE_URL +
      '/' +
      this.form.value.folderPath +
      '/cover/' +
      'side.jpg';
    this.onFrontImageUrl(front);
    this.onBackImageUrl(back);
    this.onSideImageUrl(side);

    const pdf =
      environment.DOWNLOAD_BASE_URL +
      '/' +
      this.form.value.folderPath +
      '/pdf/' +
      'book.pdf';

    const epub =
      environment.DOWNLOAD_BASE_URL +
      '/' +
      this.form.value.folderPath +
      '/pdf/' +
      'book.epub';

    // const zip =
    //   environment.DOWNLOAD_BASE_URL +
    //   '/' +
    //   this.form.value.folderPath +
    //   '/pdf/' +
    //   'book.zip';

    if (!this.form.value.pdfUrl) this.onPdfFile(pdf);
    if (!this.form.value.epubUrl) this.onEpubFile(epub);
    // this.onZipFile(zip);
  }
  // onChangeType() {
  //   const obj = this.typesList.find((t) => t.value === this.form.value.type);
  //   this.selectedType = {
  //     label: obj!.label,
  //     value: obj!.value,
  //   };
  // }

  // onImage(image: any) {
  //   console.log('img ', image);
  // }

  onFrontImageUrl(imageUrl: string) {
    this.form.patchValue({
      frontImageUrl: imageUrl,
    });
  }

  onBackImageUrl(imageUrl: string) {
    this.form.patchValue({
      backImageUrl: imageUrl,
    });
  }

  onSideImageUrl(imageUrl: string) {
    this.form.patchValue({
      sideImageUrl: imageUrl,
    });
  }

  onPdfFile(url: string) {
    this.form.patchValue({
      pdfUrl: url,
    });
  }

  onEpubFile(url: string) {
    this.form.patchValue({
      epubUrl: url,
    });
  }

  onZipFile(url: string) {
    this.form.patchValue({
      zipUrl: url,
    });
  }

  onDeleteBook() {
    this.confirmationService.confirm({
      message: `
<h3>Are you sure you want to delete this book?</h3>
      `,
      header: 'Delete Book Confirmation',
      icon: 'pi pi-info-circle',
      acceptButtonStyleClass: 'p-button-danger p-button-text',
      rejectButtonStyleClass: 'p-button-text p-button-text',
      acceptIcon: 'none',
      rejectIcon: 'none',

      accept: () => {
        this.booksService
          .deleteBook(this.bookId)
          .pipe(first())
          .subscribe((t) => {
            this.messagesService.add({
              severity: 'info',
              summary: 'Confirmed',
              detail: 'Book deleted',
            });
            this.router.navigate(['/', 'my-books']);
          });
      },
      reject: () => {},
    });
  }
}
